@@ -31,6 +31,7 @@ class KISS(object):
     def __init__(self, strip_df_start: bool=False) -> None:
         self.strip_df_start = strip_df_start
         self.interface = None
+        self.reading = False
 
     def __enter__(self):
         return self
@@ -106,7 +107,9 @@ class KISS(object):
 
         read_buffer = bytes()
 
-        while 1:
+        self.reading = True
+
+        while self.reading == True:
             read_data = self._read_handler(read_bytes)
 
             if read_data is not None and len(read_data):
@@ -213,6 +216,7 @@ class TCPKISS(KISS):
         return read_data
 
     def stop(self):
+        self.reading = False
         if self.interface:
             self.interface.shutdown(socket.SHUT_RDWR)
 
@@ -233,20 +237,51 @@ class TCPServerKISS(KISS):
     def __init__(self, host, port, strip_df_start=False):
         self.address = (host, int(port))
         self.strip_df_start = strip_df_start
-        self.conn = None
+        self.conns = []
         super(TCPServerKISS, self).__init__(strip_df_start)
+    
+    def _write_handler(self, frame):
+        conns = self.conns
+        try:
+            conn, addr = self.interface.accept()
+            self._logger.info(f"Client connected from {addr}")
+            self.conns.append(conn)
+        except socket.timeout:
+            pass
+        conns = self.conns
+        for conn in conns:
+            try:
+                conn.send(frame)
+            except ConnectionResetError:
+                self.conns.remove(conn)
+            except BrokenPipeError:
+                self.conns.remove(conn)
 
     def _read_handler(self, read_bytes=None):
-        if self.conn == None:
-            self.conn, addr = self.interface.accept()
         read_bytes = read_bytes or constants.READ_BYTES
-        read_data = self.conn.recv(read_bytes)
-        self._logger.debug('len(read_data)=%s', len(read_data))
-        if len(read_data) == 0:
-            self.conn = None
-        return read_data
+        try:
+            conn, addr = self.interface.accept()
+            self._logger.info(f"Client connected from {addr}")
+            conn.settimeout(0.01)
+            self.conns.append(conn)
+        except socket.timeout:
+            pass
+        conns = self.conns
+        for conn in self.conns:
+            try: 
+                read_data = conn.recv(read_bytes)
+                if read_data:
+                    self._logger.info(f"Read KISS bytes from TCP client : {read_bytes}")
+                    self._logger.debug('len(read_data)=%s', len(read_data))
+                    return read_data
+            except socket.timeout:
+                pass
+            except ConnectionResetError: 
+                self._logger.info(f"{conn} Disconnected")
+                self.conns.remove(conn)
 
     def stop(self):
+        self.reading = False
         if self.interface:
             self.interface.shutdown(socket.SHUT_RDWR)
 
@@ -255,10 +290,12 @@ class TCPServerKISS(KISS):
         Initializes the KISS device and commits configuration.
         """
         self.interface = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.interface.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.interface.bind(self.address)
-        self.interface.listen(0)
+        self.interface.listen(1)
+        self.interface.settimeout(0.01)
         self._logger.info('Server started')
-        self._write_handler = self.interface.send
+        #self._write_handler = self.interface.send
 
 class SerialKISS(KISS):
 
@@ -313,6 +350,7 @@ class SerialKISS(KISS):
         self.interface.write(constants.KISS_OFF)
 
     def stop(self):
+        self.reading = False
         try:
             if self.interface and self.interface.isOpen():
                 self.interface.close()
